﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.Reflection;

namespace Lab5
{
    public partial class AutoFormControl : UserControl
    {
        private object _editingObject = null;
        private Dictionary<PropertyInfo, object> _originalValues = new Dictionary<PropertyInfo, object>();

        public AutoFormControl()
        {
            InitializeComponent();
        }

        public void SetEditableObject(object editingObject)
        {
            SuspendLayout();

            //Ripulisce la collezione dei controlli
            _tableLayoutPanel.Controls.Clear();

            //Ripulisce la collezione dei valori originali
            _originalValues.Clear();

            //Setta l'oggetto di editing
            _editingObject = editingObject;

            //Ottiene il tipo dell'oggetto
            Type type = _editingObject.GetType();

            //Memorizza i valori iniziali
            foreach (PropertyInfo propertyInfo in type.GetProperties())
            {
                EditableAttribute[] attributes = (EditableAttribute[]) propertyInfo.GetCustomAttributes(typeof(EditableAttribute), false);
                if (attributes.Length == 0)
                    continue;
                if (!propertyInfo.CanRead)
                    throw new ApplicationException("L'attributo Editable non può essere applicato a proprietà write-only");

                //Memorizza solo i valori originali che possono essere sovrascritti
                if (propertyInfo.CanWrite)
                    _originalValues.Add(propertyInfo, propertyInfo.GetValue(_editingObject, null));

                AddRow(attributes[0], propertyInfo);
            }

            //Inserisce i valori di default
            RefreshValues();

            ResumeLayout(false);
        }

        public void ResetValues()
        {
            foreach (KeyValuePair<PropertyInfo, object> entry in _originalValues)
            {
                PropertyInfo propertyInfo = entry.Key;
                propertyInfo.SetValue(_editingObject, entry.Value, null);
            }
            RefreshValues();
        }

        private void AddRow(EditableAttribute editableAttribute, PropertyInfo propertyInfo)
        {
            Label label = new Label();
            label.Text = editableAttribute.Label;
            label.AutoSize = true;
            _tableLayoutPanel.Controls.Add(label);

            TextBox textBox = new TextBox();
            textBox.Size = new Size(editableAttribute.Width, textBox.PreferredHeight);
            textBox.Enabled = propertyInfo.CanWrite;
            textBox.Validating += new CancelEventHandler(ValidatingHandler);
            textBox.Tag = propertyInfo;
            _tableLayoutPanel.Controls.Add(textBox);
        }

        private void ValidatingHandler(object sender, CancelEventArgs args)
        {
            TextBox textBox = (TextBox) sender;
            PropertyInfo propertyInfo = (PropertyInfo) textBox.Tag;
            try
            {
                object convertedValue = Convert.ChangeType(textBox.Text, propertyInfo.PropertyType);
                propertyInfo.SetValue(_editingObject, convertedValue, null);
                _errorProvider.SetError(textBox, null);
                RefreshValues();
            }
            catch (Exception exception)
            {
                while (exception.InnerException != null)
                    exception = exception.InnerException;
                _errorProvider.SetError(textBox, exception.Message);
                args.Cancel = true;
            }
        }

        private void RefreshValues()
        {
            foreach (Control control in _tableLayoutPanel.Controls)
            {
                if (control is TextBox)
                {
                    TextBox textBox = (TextBox) control;
                    PropertyInfo propertyInfo = (PropertyInfo) textBox.Tag;
                    object value = propertyInfo.GetValue(_editingObject, null);
                    textBox.Text = value != null ? value.ToString() : "";
                }
            }
        }
    }
}
